@@ -5,7 +5,7 @@ use vars qw($AUTOLOAD $VERSION $DEBUG $program);
 
 use IPC::Run qw(run);
 
-$VERSION = '0.2';
+$VERSION = '0.3';
 $DEBUG = 0;
 $program = "xvcg";
 
@@ -43,7 +43,7 @@ to provide a similar interface to offer some sense of consistency.
 
 VCG is now in active development and although Graph::Writer::VCG already exists,
 this module provides a similar interface to graphviz and will be more closely tied
-into vcg as it becomes more actively developed - see James Micheal DuPonts announcement
+into vcg as it becomes more actively developed - see James Micheal DuPont's announcement
 at http://mail.gnome.org/archives/dia-list/2003-February/msg00029.html.
 
 =cut
@@ -57,33 +57,35 @@ new objects are created using the constructor method 'new'.
 This method accepts name attributes in the form :
 
 my $vcg = VCG->new(outfile=>'foo.ps')
+
 my $vcg = VCG->new(title=>'Dia Dependancies Diagram',debug=>1);
+
 my $vcg = VCG->new();
 
+my %config = ( xmax => 700, ymax=>700, program=>'vcg', x=>30, y=>30 );
+my $vcg = VCG->new(%config);
+
 =cut
 
 sub new {
   my $class = shift;
   my %config = @_;
 
-  my $self = {};
+  my $self = \%config;;
   bless($self, $class);
-  $self->{config} = \%config;
   $self->{edges} = [];
   $self->{nodes} = [];
-  $self->{xmax} ||= 700;
-  $self->{ymax} ||= 700;
-  $self->{'x'} ||= 30;
-  $self->{'y'} ||= 30;
   $self->{title} ||= "untitled";
   $self->{outfile} ||= "vcg.out";
+  $self->{program} ||= $program;
+  $self->{error} = "none - everything is fine";
 
   $DEBUG = 1 if ($config{debug});
 
   return $self;
 }
 
-=head2 add_egde
+=head2 add_edge
 
 add_edge allows you to add edges to your vcg object (edges are the lines or relationships between nodes).
 
@@ -92,6 +94,7 @@ In a Finite State Diagram, edges would represent transitions between states.
 This method accepts the source, target and colour of the edge :
 
 $vcg->add_edge( source=>'from_node', target=>'to_node');
+
 $vcg->add_edge( source=>'aaa', target=>'bbb', color=>'grey');
 
 =cut
@@ -114,7 +117,9 @@ In a Finite State Diagram, nodes would be the individual states.
 This method accepts the label, title and background colour of the node :
 
 $vcg->add_node( title=>'aaa' );
+
 $vcg->add_node( label=>'aaa' );
+
 $vcg->add_node( label=>'aaa', title=>'A', color=>'yellow' );
 
 
@@ -130,6 +135,23 @@ sub add_node {
   return 1;
 }
 
+=head2 get_vcg_version
+
+You can get the version and copyright message as a string using the vcg object (requires vcg be installed)
+
+my $version = $vcg->get_vcg_version() or die "couldn't get version : $vcg->error() \n";
+
+
+=cut
+
+sub get_vcg_version {
+  my $self = shift;
+  my $version;
+  my $error;
+  run [$self->{program}, '-version'], \undef, \$version, \$self->{error};
+  return $version;
+}
+
 
 ######################################################################################
 # generate the vcg grammar for the graph
@@ -138,9 +160,14 @@ sub _get_graph {
   my $self = shift;
   my $nodes = join ("\n",@{$self->{nodes}});
   my $edges =  join ("\n",@{$self->{edges}});
+  my $values = "";
+  foreach my $field (qw/xmax ymax x y/) {
+    $values .= "$field:$self->{$field} " if ( defined $self->{$field} );
+  }
+
   my $graph = <<end;
 graph: { title: "$self->{title}"
-xmax: $self->{xmax} ymax: $self->{ymax} x: $self->{x} y: $self->{y}
+$values
 $nodes
 $edges
 }
@@ -156,20 +183,22 @@ You can access the output in any of postscript, pbm, ppm, vcg (annotated) and vc
 
 my $image_as_ppm = $vcg->as_ppm(); # string of image as formatted as ppm
 
-
-
-my $vcg_with_coords = $vcg->as_vcg();
+my $vcg_with_coords = $vcg->as_vcg(); # handy for building a pixmap or something or converting to dia xml for example
 
 =head2 output_as_ps, output_as_pbm, output_as_ppm
 
 The VCG object allows you to output straight to a file through the vcg tool in any of postscript, pbm and ppm. This functionality requires that the vcg tool be installed.
 
-
+$vcg->output_as_ps('my_diagram.ps'); # now open the file in the gimp or import into LaTeX and you can get this free Mad Scientist (TM) white coat and bunsen burner.
 
 =head2 output_as_vcg, output_as_plainvcg
 
 The VCG object also allows you to output straight to file in annotated vcg with coordinates, or plain vcg syntax. The plain syntax does not require the vcg tool to be installed.
 
+$vcg->output_as_plainvcg('compiler_graph.vcg'); # just in case you want to generate a diagram but don't have vcg installed.
+
+$vcg->output_as_vcg('compiler_graph_with_coords.vcg'); # lovely jubbly
+
 =cut
 
 ######################################################################################
@@ -185,35 +214,54 @@ sub AUTOLOAD {
 
   return if $name =~ /DESTROY/;
 
-  if ($name =~ /^(output_)?as_(ps|pbm|ppm|vcgplain|vcg)$/) {
-    my $filename = shift || $self->{config}{outfile};
-    my $vcg = $self->_get_graph();
-    my $output;
-    my $return_file = $1;
-    my $filetype = $2;
+  my $filename = shift || $self->{outfile};
+  my $vcg = $self->_get_graph();
+  my $output;
 
-    if ($filetype eq /vcgplain/) {
-      return $vcg unless ($1);
-      open OUTFILE,">$filename" or die "couldn't open $filename for output : $!\n";
-      print OUTFILE $vcg;
-      close OUTFILE;
-      return 1;
-    }
+  if ($name =~ /^as_(ps|pbm|ppm|plainvcg|vcg)/) {
+    my $filetype = $1;
+
+    if ($filetype eq "plainvcg") {
+      $output = $vcg;
+    } else {
+      unlink $filename if (-f $filename);
+
+      if (defined $self->{scale}) {
+	run [$self->{program}, "-$filetype".'output', $filename, "-scale $self->{scale}", "- "], \$vcg, \$output;
+      } else {
+	run [$self->{program}, "-$filetype".'output', $filename, "- "], \$vcg, \$output;
+      }
+
+      warn $output if ($DEBUG);
 
-    run [$program, "-$filetype".'output', $filename, "- "], \$vcg, \$output;
-    warn $output if ($DEBUG);
-    unless ($1) {
       open (FILE,$filename) or die "unable to open $filename : $!\n";
       my $data = join ('',(<FILE>));
       close FILE;
-      return $data;
-      unlink $filename or die "unable to remove tempory file $filename : $! \n";
+      if (-f $filename) { unlink $filename or die "unable to remove tempory file $filename : $! \n"; }
+      $output = $data;
+    }
+  } elsif ($name =~ /output_as_(ps|pbm|ppm|plainvcg|vcg)$/){
+    my $filetype = $1;
+    if ($filetype eq "plainvcg") {
+      open OUTFILE,">$filename" or die "couldn't open $filename for output : $!\n";
+      print OUTFILE $vcg;
+      close OUTFILE;
+      $output = 1;
+    } else {
+      unlink $filename if (-f $filename);
+      if (defined $self->{scale}) {
+	run [$self->{program}, "-$filetype".'output', $filename, "-scale $self->{scale}", "- "], \$vcg, \$output;
+      } else {
+	run [$self->{program}, "-$filetype".'output', $filename, "- "], \$vcg, \$output;
+      }
     }
     return $output;
+  } else {
+    die "Method $name not defined!";
   }
-  die "Method $name not defined!";
 }
 
+
 ##########################################################################
 
 =head1 SEE ALSO
@@ -224,6 +272,8 @@ sub AUTOLOAD {
 
   Graph::Writer::VCG perl module
 
+  vcg/xvcg : man pages
+
 =head1 AUTHOR
 
 Aaron Trevena E<lt>F<aaron@droogs.org>E<gt>
